<?php
namespace Magenest\Salesforce\Controller\Adminhtml\Queue;

use Magenest\Salesforce\Helper\Data;
use Magenest\Salesforce\Model\Queue;
use Magento\Customer\Api\CustomerRepositoryInterface;
use Magento\Framework\Api\FilterBuilder;
use Magento\Sales\Api\OrderRepositoryInterface;

/**
 * Class AbstractionAction
 * @package Magenest\Salesforce\Controller\Adminhtml\Queue
 */
abstract class AbstractionAction extends \Magento\Backend\App\Action
{
    /**
     * Authorization level of a basic admin session
     */
    const ADMIN_RESOURCE = 'Magenest_Salesforce::config_salesforce';

    const ENTITY_COLUMN = 'entity_id';

    const SIZE = 500;

    /** @var \Magento\Framework\App\ResourceConnection  */
    protected $_resourceConnection;

    /** @var \Magento\Framework\MessageQueue\PublisherInterface  */
    protected $_publisher;
    /** @var \Magenest\Salesforce\Logger\Logger  */
    protected $_logger;

    /** @var Data  */
    protected $_helperData;

    /** @var \Magenest\Salesforce\Model\QueueFactory  */
    protected $queueFactory;

    /** @var \Magenest\Salesforce\Model\ResourceModel\Queue  */
    protected $_queueResource;

    /** @var \Magenest\Salesforce\Model\ResourceModel\Queue\CollectionFactory  */
    protected $_queueCollectionFactory;

    /** @var \Magento\Framework\View\Result\PageFactory  */
    protected $resultPageFactory;

    /** @var \Magento\Framework\App\Config\ScopeConfigInterface  */
    protected $_configInterface;

    /** @var \Magento\Config\Model\ResourceModel\Config  */
    protected $_config;

    /** @var \Magento\Customer\Model\ResourceModel\Customer\CollectionFactory  */
    protected $customerCollection;

    /** @var \Magento\Framework\Serialize\Serializer\Json  */
    protected $_json;

    /** @var \Magento\CatalogRule\Model\ResourceModel\Rule\CollectionFactory */
    protected $_ruleColFactory;

    /** @var \Magento\Sales\Model\ResourceModel\Order\CollectionFactory  */
    protected $_orderColFactory;

    /** @var OrderRepositoryInterface  */
    protected $_orderRepository;

    /** @var \Magento\Framework\Api\SearchCriteriaBuilder  */
    protected $_searchCriteriaBuilder;

    /** @var \Magento\Catalog\Model\ResourceModel\Product\CollectionFactory  */
    protected $_productColFactory;

    /** @var FilterBuilder  */
    protected $_filterBuilder;

    /** @var CustomerRepositoryInterface  */
    protected $_customerRepository;

    /** @var \Magento\Newsletter\Model\ResourceModel\Subscriber\CollectionFactory */
    protected $subscriberCollectionFactory;

    /**
     * AbstractionAction constructor.
     * @param \Magento\Framework\App\ResourceConnection $resourceConnection
     * @param \Magento\Framework\MessageQueue\PublisherInterface $publisher
     * @param \Magenest\Salesforce\Logger\Logger $logger
     * @param Data $helperData
     * @param \Magenest\Salesforce\Model\ResourceModel\Queue $queueResource
     * @param \Magenest\Salesforce\Model\ResourceModel\Queue\CollectionFactory $queueCollectionFactory
     * @param \Magento\Newsletter\Model\ResourceModel\Subscriber\CollectionFactory $subscriberCollectionFactory
     * @param \Magento\Framework\View\Result\PageFactory $resultPageFactory
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
     * @param \Magento\Config\Model\ResourceModel\Config $config
     * @param \Magento\Customer\Model\ResourceModel\Customer\CollectionFactory $customerCollection
     * @param \Magento\Framework\Serialize\Serializer\Json $json
     * @param \Magento\CatalogRule\Model\ResourceModel\Rule\CollectionFactory $ruleColFactory
     * @param \Magento\Sales\Model\ResourceModel\Order\CollectionFactory $orderColFactory
     * @param OrderRepositoryInterface $orderRepository
     * @param \Magento\Framework\Api\SearchCriteriaBuilder $searchCriteriaBuilder
     * @param \Magento\Catalog\Model\ResourceModel\Product\CollectionFactory $productColFactory
     * @param FilterBuilder $filterBuilder
     * @param CustomerRepositoryInterface $customerRepository
     * @param \Magento\Backend\App\Action\Context $context
     */
    public function __construct(
        \Magento\Framework\App\ResourceConnection $resourceConnection,
        \Magento\Framework\MessageQueue\PublisherInterface $publisher,
        \Magenest\Salesforce\Logger\Logger $logger,
        \Magenest\Salesforce\Helper\Data $helperData,
        \Magenest\Salesforce\Model\ResourceModel\Queue $queueResource,
        \Magenest\Salesforce\Model\ResourceModel\Queue\CollectionFactory $queueCollectionFactory,
        \Magento\Newsletter\Model\ResourceModel\Subscriber\CollectionFactory $subscriberCollectionFactory,
        \Magento\Framework\View\Result\PageFactory $resultPageFactory,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Magento\Config\Model\ResourceModel\Config $config,
        \Magento\Customer\Model\ResourceModel\Customer\CollectionFactory $customerCollection,
        \Magento\Framework\Serialize\Serializer\Json $json,
        \Magento\CatalogRule\Model\ResourceModel\Rule\CollectionFactory $ruleColFactory,
        \Magento\Sales\Model\ResourceModel\Order\CollectionFactory $orderColFactory,
        \Magento\Sales\Api\OrderRepositoryInterface $orderRepository,
        \Magento\Framework\Api\SearchCriteriaBuilder $searchCriteriaBuilder,
        \Magento\Catalog\Model\ResourceModel\Product\CollectionFactory $productColFactory,
        \Magento\Framework\Api\FilterBuilder $filterBuilder,
        CustomerRepositoryInterface $customerRepository,
        \Magento\Backend\App\Action\Context $context
    ) {
        $this->_resourceConnection = $resourceConnection;
        $this->_publisher = $publisher;
        $this->_logger = $logger;
        $this->_helperData = $helperData;
        $this->_queueResource = $queueResource;
        $this->_queueCollectionFactory = $queueCollectionFactory;
        $this->resultPageFactory = $resultPageFactory;
        $this->_configInterface = $scopeConfig;
        $this->_config = $config;
        $this->customerCollection = $customerCollection;
        $this->_json = $json;
        $this->_ruleColFactory = $ruleColFactory;
        $this->_orderColFactory = $orderColFactory;
        $this->_orderRepository = $orderRepository;
        $this->_searchCriteriaBuilder = $searchCriteriaBuilder;
        $this->_productColFactory = $productColFactory;
        $this->_filterBuilder = $filterBuilder;
        $this->_customerRepository = $customerRepository;
        $this->subscriberCollectionFactory = $subscriberCollectionFactory;
        parent::__construct($context);
    }

    /**
     * @param $data
     * @param $type
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    public function publishData($data, $type)
    {
        $this->_queueResource->publishData($data, $type);
    }

    /**
     * @param $type
     *
     * @return mixed
     * @throws \Exception
     */
    public function getQueueIdsByType($type)
    {
        if (is_string($type)) {
            $queueIds = $this->_queueCollectionFactory->create()->addFieldToFilter('type', $type)->getAllIds();
        } else {
            throw new \Exception(__('type variable is not string'));
        }
        return $queueIds;
    }

    /**
     * @param $collection
     * @param string $fromField
     * @param string $toField
     * @param string $type
     * @return mixed
     * @throws \Exception
     */
    protected function getFilterDate($collection, $fromField = 'created_at', $toField = 'created_at', $type = '')
    {
        try {
            $params = $this->getRequest()->getParams();
            $fromDate = $params['from'] ?? null;
            $toDate = $params['to'] ?? null;
            if ($type != Queue::TYPE_CAMPAIGN) {
                $fromDate = $fromDate != null ? $fromDate : '2000-01-01';
                $toDate = $toDate != null ? $toDate : '2099-01-01';
                $fromDate = $fromDate . ' 00:00:00';
                $toDate = $toDate . ' 23:59:59';
            }
            if ($fromDate != null) {
                $collection->addFieldToFilter($fromField, ['gteq' => $fromDate]);
            }
            if ($toDate != null) {
                $collection->addFieldToFilter($toField, ['lteq' => $toDate]);
            }
            return $collection;
        } catch (\Throwable $exception) {
            throw new \Exception($exception->getMessage());
        }
    }
}
