<?php
namespace Magenest\Salesforce\Model\Config\Source;

use Magenest\Salesforce\Logger\Logger;
use Magenest\Salesforce\Model\Connector;
use Magenest\Salesforce\Model\FieldFactory;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\Data\OptionSourceInterface;

class AccountFields implements OptionSourceInterface
{
    /** @var array|null */
    private $options = null;

    /** @var FieldFactory */
    private $fieldFactory;

    /** @var ScopeConfigInterface */
    private $scopeConfig;

    /**
     * @param FieldFactory $fieldFactory
     * @param ScopeConfigInterface $scopeConfig
     */
    public function __construct(FieldFactory $fieldFactory, ScopeConfigInterface $scopeConfig, Logger $logger)
    {
        $this->logger       = $logger;
        $this->scopeConfig  = $scopeConfig;
        $this->fieldFactory = $fieldFactory;
    }

    /**
     * Return options array
     *
     * @return array
     * @throws \Exception
     */
    public function toOptionArray()
    {
        if ($this->options === null) {
            $this->options = ["Disable"];
            if ($this->scopeConfig->getValue(Connector::XML_PATH_SALESFORCE_IS_CONNECTED)) {
                try {
                    $this->options += $this->fieldFactory->create()->getUnmapFieldData('Account');
                } catch (\Throwable $e) {
                    $this->logger->critical($e->getMessage(), ['trace' => $e->getTraceAsString()]);
                }

            }
        }

        return $this->options;
    }
}
